@@ -1,14 +1,39 @@
-import os
-import django
+from datetime import datetime
 from django.conf import settings
+from django.core.cache import get_cache
 from rosetta.conf import settings as rosetta_settings
-from django.core.cache import cache
+import django
+import os
+
+try:
+    from django.utils import timezone
+except:
+    timezone = None
+
 
 try:
     set
 except NameError:
     from sets import Set as set   # Python 2.3 fallback
 
+cache = get_cache(rosetta_settings.ROSETTA_CACHE_NAME)
+
+
+def timestamp_with_timezone(dt=None):
+    """
+    Return a timestamp with a timezone for the configured locale.  If all else
+    fails, consider localtime to be UTC.
+    """
+    dt = dt or datetime.now()
+    if timezone is None:
+        return dt.strftime('%Y-%m-%d %H:%M%z')
+    if not dt.tzinfo:
+        tz = timezone.get_current_timezone()
+        if not tz:
+            tz = timezone.utc
+        dt = dt.replace(tzinfo=timezone.get_current_timezone())
+    return dt.strftime("%Y-%m-%d %H:%M%z")
+
 
 def find_pos(lang, project_apps=True, django_apps=False, third_party_apps=False):
     """
@@ -24,7 +49,10 @@ def find_pos(lang, project_apps=True, django_apps=False, third_party_apps=False)
     project = __import__(parts[0], {}, {}, [])
     abs_project_path = os.path.normpath(os.path.abspath(os.path.dirname(project.__file__)))
     if project_apps:
-        paths.append(os.path.abspath(os.path.join(os.path.dirname(project.__file__), 'locale')))
+        if os.path.exists(os.path.abspath(os.path.join(os.path.dirname(project.__file__), 'locale'))):
+            paths.append(os.path.abspath(os.path.join(os.path.dirname(project.__file__), 'locale')))
+        if os.path.exists(os.path.abspath(os.path.join(os.path.dirname(project.__file__), '..', 'locale'))):
+            paths.append(os.path.abspath(os.path.join(os.path.dirname(project.__file__), '..', 'locale')))
 
     # django/locale
     if django_apps:
@@ -48,12 +76,11 @@ def find_pos(lang, project_apps=True, django_apps=False, third_party_apps=False)
             continue
         p = appname.rfind('.')
         if p >= 0:
-            app = getattr(__import__(appname[:p], {}, {}, [appname[p + 1:]]), appname[p + 1:])
+            app = getattr(__import__(appname[:p], {}, {}, [str(appname[p + 1:])]), appname[p + 1:])
         else:
             app = __import__(appname, {}, {}, [])
 
         apppath = os.path.normpath(os.path.abspath(os.path.join(os.path.dirname(app.__file__), 'locale')))
-        
 
         # django apps
         if 'contrib' in apppath and 'django' in apppath and not django_apps:
@@ -62,52 +89,58 @@ def find_pos(lang, project_apps=True, django_apps=False, third_party_apps=False)
         # third party external
         if not third_party_apps and abs_project_path not in apppath:
             continue
-            
+
         # local apps
         if not project_apps and abs_project_path in apppath:
             continue
-            
-        
+
         if os.path.isdir(apppath):
             paths.append(apppath)
-            
-            
-        
-            
+
     ret = set()
-    langs = (lang,)
+    langs = [lang, ]
     if u'-' in lang:
-        _l,_c =  map(lambda x:x.lower(),lang.split(u'-'))
-        langs += (u'%s_%s' %(_l, _c), u'%s_%s' %(_l, _c.upper()), )
+        _l, _c = map(lambda x: x.lower(), lang.split(u'-'))
+        langs += [u'%s_%s' % (_l, _c), u'%s_%s' % (_l, _c.upper()), ]
     elif u'_' in lang:
-        _l,_c = map(lambda x:x.lower(),lang.split(u'_'))
-        langs += (u'%s-%s' %(_l, _c), u'%s-%s' %(_l, _c.upper()), )
-        
+        _l, _c = map(lambda x: x.lower(), lang.split(u'_'))
+        langs += [u'%s-%s' % (_l, _c), u'%s-%s' % (_l, _c.upper()), ]
+
     paths = map(os.path.normpath, paths)
+    paths = list(set(paths))
     for path in paths:
         for lang_ in langs:
             dirname = os.path.join(path, lang_, 'LC_MESSAGES')
-            for fn in ('django.po','djangojs.po',):
+            #print dirname
+            for fn in ('django.po', 'djangojs.po',):
                 filename = os.path.join(dirname, fn)
                 if os.path.isfile(filename):
                     ret.add(os.path.abspath(filename))
-    return list(ret)
+    return list(sorted(ret))
 
-def pagination_range(first,last,current):
+
+def pagination_range(first, last, current):
     r = []
-    
+
     r.append(first)
-    if first + 1 < last: r.append(first+1)
-    
-    if current -2 > first and current -2 < last: r.append(current-2)
-    if current -1 > first and current -1 < last: r.append(current-1)
-    if current > first and current < last: r.append(current)
-    if current + 1 < last and current+1 > first: r.append(current+1)
-    if current + 2 < last and current+2 > first: r.append(current+2)
-    
-    if last-1 > first: r.append(last-1)
+    if first + 1 < last:
+        r.append(first + 1)
+
+    if current - 2 > first and current - 2 < last:
+        r.append(current - 2)
+    if current - 1 > first and current - 1 < last:
+        r.append(current - 1)
+    if current > first and current < last:
+        r.append(current)
+    if current + 1 < last and current + 1 > first:
+        r.append(current + 1)
+    if current + 2 < last and current + 2 > first:
+        r.append(current + 2)
+
+    if last - 1 > first:
+        r.append(last - 1)
     r.append(last)
-    
+
     r = list(set(r))
     r.sort()
     prev = 10000
