@@ -5,21 +5,25 @@ from django.core.urlresolvers import reverse
 from django.http import Http404, HttpResponseRedirect, HttpResponse
 from django.shortcuts import render_to_response
 from django.template import RequestContext
-from django.utils.encoding import smart_unicode, iri_to_uri
+from django.utils.encoding import iri_to_uri
 from django.utils.translation import ugettext_lazy as _
 from django.views.decorators.cache import never_cache
 from rosetta.conf import settings as rosetta_settings
 from rosetta.polib import pofile
-from rosetta.poutil import find_pos, pagination_range
+from rosetta.poutil import find_pos, pagination_range, timestamp_with_timezone
 from rosetta.signals import entry_changed, post_save
+from rosetta.storage import get_storage
+from rosetta.access import can_translate
 import re
 import rosetta
-import datetime
 import unicodedata
 import hashlib
 import os
+import six
 
 
+@never_cache
+@user_passes_test(lambda user: can_translate(user), settings.LOGIN_URL)
 def home(request):
     """
     Displays a list of messages to be translated
@@ -45,28 +49,33 @@ def home(request):
             out_ = out_.rstrip()
         return out_
 
-    version = rosetta.get_version(True)
-    if 'rosetta_i18n_fn' in request.session:
-        rosetta_i18n_fn = request.session.get('rosetta_i18n_fn')
+    storage = get_storage(request)
+    query = ''
+    if storage.has('rosetta_i18n_fn'):
+        rosetta_i18n_fn = storage.get('rosetta_i18n_fn')
         rosetta_i18n_app = get_app_name(rosetta_i18n_fn)
-        rosetta_i18n_lang_code = request.session['rosetta_i18n_lang_code']
+        rosetta_i18n_lang_code = storage.get('rosetta_i18n_lang_code')
         rosetta_i18n_lang_bidi = rosetta_i18n_lang_code.split('-')[0] in settings.LANGUAGES_BIDI
-        rosetta_i18n_write = request.session.get('rosetta_i18n_write', True)
+        rosetta_i18n_write = storage.get('rosetta_i18n_write', True)
         if rosetta_i18n_write:
             rosetta_i18n_pofile = pofile(rosetta_i18n_fn, wrapwidth=rosetta_settings.POFILE_WRAP_WIDTH)
             for entry in rosetta_i18n_pofile:
-                entry.md5hash = hashlib.md5(entry.msgid.encode("utf8") + entry.msgstr.encode("utf8")).hexdigest()
+                entry.md5hash = hashlib.md5(
+                    (six.text_type(entry.msgid) +
+                    six.text_type(entry.msgstr) +
+                    six.text_type(entry.msgctxt or "")).encode('utf8')
+                ).hexdigest()
 
         else:
-            rosetta_i18n_pofile = request.session.get('rosetta_i18n_pofile')
+            rosetta_i18n_pofile = storage.get('rosetta_i18n_pofile')
 
         if 'filter' in request.GET:
             if request.GET.get('filter') in ('untranslated', 'translated', 'fuzzy', 'all'):
                 filter_ = request.GET.get('filter')
-                request.session['rosetta_i18n_filter'] = filter_
+                storage.set('rosetta_i18n_filter', filter_)
                 return HttpResponseRedirect(reverse('rosetta-home'))
 
-        rosetta_i18n_filter = request.session.get('rosetta_i18n_filter', 'all')
+        rosetta_i18n_filter = storage.get('rosetta_i18n_filter', 'all')
 
         if '_next' in request.POST:
             rx = re.compile(r'^m_([0-9a-f]+)')
@@ -81,7 +90,7 @@ def home(request):
                     # polib parses .po files into unicode strings, but
                     # doesn't bother to convert plural indexes to int,
                     # so we need unicode here.
-                    plural_id = unicode(rx_plural.match(key).groups()[1])
+                    plural_id = six.text_type(rx_plural.match(key).groups()[1])
 
                 elif rx.match(key):
                     md5hash = str(rx.match(key).groups()[0])
@@ -93,7 +102,8 @@ def home(request):
                     if entry:
                         old_msgstr = entry.msgstr
                         if plural_id is not None:
-                            plural_string = fix_nls(entry.msgstr_plural[plural_id], value)
+                            #plural_string = fix_nls(entry.msgstr_plural[plural_id], value)
+                            plural_string = fix_nls(entry.msgid_plural, value)
                             entry.msgstr_plural[plural_id] = plural_string
                         else:
                             entry.msgstr = fix_nls(entry.msgid, value)
@@ -118,7 +128,7 @@ def home(request):
                                                )
 
                     else:
-                        request.session['rosetta_last_save_error'] = True
+                        storage.set('rosetta_last_save_error', True)
 
             if file_change and rosetta_i18n_write:
                 try:
@@ -129,12 +139,15 @@ def home(request):
 
                     rosetta_i18n_pofile.metadata['Last-Translator'] = unicodedata.normalize('NFKD', u"%s %s <%s>" % (request.user.first_name, request.user.last_name, request.user.email)).encode('ascii', 'ignore')
                     rosetta_i18n_pofile.metadata['X-Translated-Using'] = u"django-rosetta %s" % rosetta.get_version(False)
-                    rosetta_i18n_pofile.metadata['PO-Revision-Date'] = datetime.datetime.now().strftime('%Y-%m-%d %H:%M%z')
+                    rosetta_i18n_pofile.metadata['PO-Revision-Date'] = timestamp_with_timezone()
                 except UnicodeDecodeError:
                     pass
+
                 try:
                     rosetta_i18n_pofile.save()
-                    rosetta_i18n_pofile.save_as_mofile(rosetta_i18n_fn.replace('.po', '.mo'))
+                    po_filepath, ext = os.path.splitext(rosetta_i18n_fn)
+                    save_as_mo_filepath = po_filepath + '.mo'
+                    rosetta_i18n_pofile.save_as_mofile(save_as_mo_filepath)
 
                     post_save.send(sender=None, language_code=rosetta_i18n_lang_code, request=request)
                     # Try auto-reloading via the WSGI daemon mode reload mechanism
@@ -158,28 +171,22 @@ def home(request):
                             pass
 
                 except:
-                    request.session['rosetta_i18n_write'] = False
-
-                request.session['rosetta_i18n_pofile'] = rosetta_i18n_pofile
+                    storage.set('rosetta_i18n_write', False)
+                storage.set('rosetta_i18n_pofile', rosetta_i18n_pofile)
 
                 # Retain query arguments
-                query_arg = ''
-                if 'query' in request.REQUEST:
-                    query_arg = '?query=%s' % request.REQUEST.get('query')
+                query_arg = '?_next=1'
+                if 'query' in request.GET or 'query' in request.POST:
+                    query_arg += '&query=%s' % request.REQUEST.get('query')
                 if 'page' in request.GET:
-                    if query_arg:
-                        query_arg = query_arg + '&'
-                    else:
-                        query_arg = '?'
-                    query_arg = query_arg + 'page=%d' % int(request.GET.get('page'))
+                    query_arg += '&page=%d&_next=1' % int(request.GET.get('page'))
                 return HttpResponseRedirect(reverse('rosetta-home') + iri_to_uri(query_arg))
-        rosetta_i18n_lang_name = _(request.session.get('rosetta_i18n_lang_name'))
-        rosetta_i18n_lang_code = request.session.get('rosetta_i18n_lang_code')
+        rosetta_i18n_lang_code = storage.get('rosetta_i18n_lang_code')
 
         if 'query' in request.REQUEST and request.REQUEST.get('query', '').strip():
             query = request.REQUEST.get('query').strip()
             rx = re.compile(re.escape(query), re.IGNORECASE)
-            paginator = Paginator([e for e in rosetta_i18n_pofile if not e.obsolete and rx.search(smart_unicode(e.msgstr) + smart_unicode(e.msgid) + u''.join([o[0] for o in e.occurrences]))], rosetta_settings.MESSAGES_PER_PAGE)
+            paginator = Paginator([e for e in rosetta_i18n_pofile if not e.obsolete and rx.search(six.text_type(e.msgstr) + six.text_type(e.msgid) + u''.join([o[0] for o in e.occurrences]))], rosetta_settings.MESSAGES_PER_PAGE)
         else:
             if rosetta_i18n_filter == 'untranslated':
                 paginator = Paginator(rosetta_i18n_pofile.untranslated_entries(), rosetta_settings.MESSAGES_PER_PAGE)
@@ -194,10 +201,17 @@ def home(request):
             page = int(request.GET.get('page'))
         else:
             page = 1
-        rosetta_messages = paginator.page(page).object_list
-        if rosetta_settings.MAIN_LANGUAGE and rosetta_settings.MAIN_LANGUAGE != rosetta_i18n_lang_code:
 
-            main_language = None
+        if '_next' in request.GET or '_next' in request.POST:
+            page += 1
+            if page > paginator.num_pages:
+                page = 1
+            query_arg = '?page=%d' % page
+            return HttpResponseRedirect(reverse('rosetta-home') + iri_to_uri(query_arg))
+
+        rosetta_messages = paginator.page(page).object_list
+        main_language = None
+        if rosetta_settings.MAIN_LANGUAGE and rosetta_settings.MAIN_LANGUAGE != rosetta_i18n_lang_code:
             for language in settings.LANGUAGES:
                 if language[0] == rosetta_settings.MAIN_LANGUAGE:
                     main_language = _(language[1])
@@ -206,7 +220,6 @@ def home(request):
             fl = ("/%s/" % rosetta_settings.MAIN_LANGUAGE).join(rosetta_i18n_fn.split("/%s/" % rosetta_i18n_lang_code))
             po = pofile(fl)
 
-            main_messages = []
             for message in rosetta_messages:
                 message.main_lang = po.find(message.msgid).msgstr
 
@@ -222,30 +235,49 @@ def home(request):
         except AttributeError:
             ADMIN_MEDIA_PREFIX = settings.STATIC_URL + 'admin/'
             ADMIN_IMAGE_DIR = ADMIN_MEDIA_PREFIX + 'img/'
-        ENABLE_TRANSLATION_SUGGESTIONS = rosetta_settings.BING_APP_ID and rosetta_settings.ENABLE_TRANSLATION_SUGGESTIONS
-        BING_APP_ID = rosetta_settings.BING_APP_ID
-        MESSAGES_SOURCE_LANGUAGE_NAME = rosetta_settings.MESSAGES_SOURCE_LANGUAGE_NAME
-        MESSAGES_SOURCE_LANGUAGE_CODE = rosetta_settings.MESSAGES_SOURCE_LANGUAGE_CODE
-        if 'rosetta_last_save_error' in request.session:
-            del(request.session['rosetta_last_save_error'])
+
+        if storage.has('rosetta_last_save_error'):
+            storage.delete('rosetta_last_save_error')
             rosetta_last_save_error = True
+        else:
+            rosetta_last_save_error = False
 
-        return render_to_response('rosetta/pofile.html', locals(), context_instance=RequestContext(request))
+        return render_to_response('rosetta/pofile.html', dict(
+            version=rosetta.get_version(True),
+            ADMIN_MEDIA_PREFIX=ADMIN_MEDIA_PREFIX,
+            ADMIN_IMAGE_DIR=ADMIN_IMAGE_DIR,
+            rosetta_settings=rosetta_settings,
+            rosetta_i18n_lang_name=_(storage.get('rosetta_i18n_lang_name')),
+            rosetta_i18n_lang_code=rosetta_i18n_lang_code,
+            rosetta_i18n_lang_bidi=rosetta_i18n_lang_bidi,
+            rosetta_last_save_error=rosetta_last_save_error,
+            rosetta_i18n_filter=rosetta_i18n_filter,
+            rosetta_i18n_write=rosetta_i18n_write,
+            rosetta_messages=rosetta_messages,
+            page_range=needs_pagination and page_range,
+            needs_pagination=needs_pagination,
+            main_language=main_language,
+            rosetta_i18n_app=rosetta_i18n_app,
+            page=page,
+            query=query,
+            paginator=paginator,
+            rosetta_i18n_pofile=rosetta_i18n_pofile
+        ), context_instance=RequestContext(request))
     else:
-        return list_languages(request)
-home = never_cache(home)
-home = user_passes_test(lambda user: can_translate(user), settings.LOGIN_URL)(home)
+        return list_languages(request, do_session_warn=True)
 
 
+@never_cache
+@user_passes_test(lambda user: can_translate(user), settings.LOGIN_URL)
 def download_file(request):
     import zipfile
-    from StringIO import StringIO
+    storage = get_storage(request)
     # original filename
-    rosetta_i18n_fn = request.session.get('rosetta_i18n_fn', None)
+    rosetta_i18n_fn = storage.get('rosetta_i18n_fn', None)
     # in-session modified catalog
-    rosetta_i18n_pofile = request.session.get('rosetta_i18n_pofile', None)
+    rosetta_i18n_pofile = storage.get('rosetta_i18n_pofile', None)
     # language code
-    rosetta_i18n_lang_code = request.session.get('rosetta_i18n_lang_code', None)
+    rosetta_i18n_lang_code = storage.get('rosetta_i18n_lang_code', None)
 
     if not rosetta_i18n_lang_code or not rosetta_i18n_pofile or not rosetta_i18n_fn:
         return HttpResponseRedirect(reverse('rosetta-home'))
@@ -256,9 +288,9 @@ def download_file(request):
             offered_fn = rosetta_i18n_fn.split('/')[-1]
         po_fn = str(rosetta_i18n_fn.split('/')[-1])
         mo_fn = str(po_fn.replace('.po', '.mo'))  # not so smart, huh
-        zipdata = StringIO()
+        zipdata = six.BytesIO()
         zipf = zipfile.ZipFile(zipdata, mode="w")
-        zipf.writestr(po_fn, unicode(rosetta_i18n_pofile).encode("utf8"))
+        zipf.writestr(po_fn, six.text_type(rosetta_i18n_pofile).encode("utf8"))
         zipf.writestr(mo_fn, rosetta_i18n_pofile.to_binary())
         zipf.close()
         zipdata.seek(0)
@@ -270,24 +302,25 @@ def download_file(request):
 
     except Exception:
         return HttpResponseRedirect(reverse('rosetta-home'))
-download_file = never_cache(download_file)
-download_file = user_passes_test(lambda user: can_translate(user), settings.LOGIN_URL)(download_file)
 
 
-def list_languages(request):
+@never_cache
+@user_passes_test(lambda user: can_translate(user), settings.LOGIN_URL)
+def list_languages(request, do_session_warn=False):
     """
     Lists the languages for the current project, the gettext catalog files
     that can be translated and their translation progress
     """
+    storage = get_storage(request)
     languages = []
 
     if 'filter' in request.GET:
         if request.GET.get('filter') in ('project', 'third-party', 'django', 'all'):
             filter_ = request.GET.get('filter')
-            request.session['rosetta_i18n_catalog_filter'] = filter_
+            storage.set('rosetta_i18n_catalog_filter', filter_)
             return HttpResponseRedirect(reverse('rosetta-pick-file'))
 
-    rosetta_i18n_catalog_filter = request.session.get('rosetta_i18n_catalog_filter', 'project')
+    rosetta_i18n_catalog_filter = storage.get('rosetta_i18n_catalog_filter', 'project')
 
     third_party_apps = rosetta_i18n_catalog_filter in ('all', 'third-party')
     django_apps = rosetta_i18n_catalog_filter in ('all', 'django')
@@ -300,18 +333,22 @@ def list_languages(request):
         languages.append(
             (language[0],
             _(language[1]),
-            [(get_app_name(l), os.path.realpath(l), pofile(l)) for l in  pos],
+            sorted([(get_app_name(l), os.path.realpath(l), pofile(l)) for l in  pos], key=lambda app: app[0]),
             )
         )
     try:
         ADMIN_MEDIA_PREFIX = settings.ADMIN_MEDIA_PREFIX
     except AttributeError:
         ADMIN_MEDIA_PREFIX = settings.STATIC_URL + 'admin/'
+    do_session_warn = do_session_warn and 'SessionRosettaStorage' in rosetta_settings.STORAGE_CLASS and 'signed_cookies' in settings.SESSION_ENGINE
 
-    version = rosetta.get_version(True)
-    return render_to_response('rosetta/languages.html', locals(), context_instance=RequestContext(request))
-list_languages = never_cache(list_languages)
-list_languages = user_passes_test(lambda user: can_translate(user), settings.LOGIN_URL)(list_languages)
+    return render_to_response('rosetta/languages.html', dict(
+        version=rosetta.get_version(True),
+        ADMIN_MEDIA_PREFIX=ADMIN_MEDIA_PREFIX,
+        do_session_warn=do_session_warn,
+        languages=languages,
+        has_pos=has_pos
+    ), context_instance=RequestContext(request))
 
 
 def get_app_name(path):
@@ -319,52 +356,40 @@ def get_app_name(path):
     return app
 
 
+@never_cache
+@user_passes_test(lambda user: can_translate(user), settings.LOGIN_URL)
 def lang_sel(request, langid, idx):
     """
     Selects a file to be translated
     """
+    storage = get_storage(request)
     if langid not in [l[0] for l in settings.LANGUAGES]:
         raise Http404
     else:
 
-        rosetta_i18n_catalog_filter = request.session.get('rosetta_i18n_catalog_filter', 'project')
+        rosetta_i18n_catalog_filter = storage.get('rosetta_i18n_catalog_filter', 'project')
 
         third_party_apps = rosetta_i18n_catalog_filter in ('all', 'third-party')
         django_apps = rosetta_i18n_catalog_filter in ('all', 'django')
         project_apps = rosetta_i18n_catalog_filter in ('all', 'project')
-        file_ = find_pos(langid, project_apps=project_apps, django_apps=django_apps, third_party_apps=third_party_apps)[int(idx)]
+        file_ = sorted(find_pos(langid, project_apps=project_apps, django_apps=django_apps, third_party_apps=third_party_apps), key=get_app_name)[int(idx)]
 
-        request.session['rosetta_i18n_lang_code'] = langid
-        request.session['rosetta_i18n_lang_name'] = unicode([l[1] for l in settings.LANGUAGES if l[0] == langid][0])
-        request.session['rosetta_i18n_fn'] = file_
+        storage.set('rosetta_i18n_lang_code', langid)
+        storage.set('rosetta_i18n_lang_name', six.text_type([l[1] for l in settings.LANGUAGES if l[0] == langid][0]))
+        storage.set('rosetta_i18n_fn',  file_)
         po = pofile(file_)
         for entry in po:
-            entry.md5hash = hashlib.md5(entry.msgid.encode("utf8") + entry.msgstr.encode("utf8")).hexdigest()
+            entry.md5hash = hashlib.new('md5',
+                (six.text_type(entry.msgid) +
+                six.text_type(entry.msgstr) +
+                six.text_type(entry.msgctxt or "")).encode('utf8')
+            ).hexdigest()
 
-        request.session['rosetta_i18n_pofile'] = po
+        storage.set('rosetta_i18n_pofile', po)
         try:
             os.utime(file_, None)
-            request.session['rosetta_i18n_write'] = True
+            storage.set('rosetta_i18n_write', True)
         except OSError:
-            request.session['rosetta_i18n_write'] = False
+            storage.set('rosetta_i18n_write', False)
 
         return HttpResponseRedirect(reverse('rosetta-home'))
-lang_sel = never_cache(lang_sel)
-lang_sel = user_passes_test(lambda user: can_translate(user), settings.LOGIN_URL)(lang_sel)
-
-
-def can_translate(user):
-    if not getattr(settings, 'ROSETTA_REQUIRES_AUTH', True):
-        return True
-    if not user.is_authenticated():
-        return False
-    elif user.is_superuser and user.is_staff:
-        return True
-    else:
-        try:
-            from django.contrib.auth.models import Group
-            translators = Group.objects.get(name='translators')
-            return translators in user.groups.all()
-        except Group.DoesNotExist:
-            return False
-
